(proclaim '(optimize (speed 0) (safety 3) (debug 3)))

(defparameter *output* nil)
(defparameter *command-step* 0)

;;; creates x"AD" from #xAD
(defun number-to-hex (number)
  (format nil "x\"~2,'0X\"" number))

(defun test-z-flag (register)
  (emit "if" register "= x\"00\" then"
        "z_flag <= '1';"
        "else"
        "z_flag <= '0';"
        "end if;"))

(defun test-n-flag (bit)
  (emit "if" bit "= '1' then"
        "n_flag <= '1';"
        "else"
        "n_flag <= '0';"
        "end if;"))

(defun emit (&rest objects)
  (loop for object in objects do
        (format *output* "~a " object))
  (terpri *output*))

(defun emit-lines (&rest objects)
  (loop for object in objects do
        (format *output* "~a~%" object)))

(defun command (name code)
  (emit "--" name)
  (emit "when" (number-to-hex code) "=>")
  (emit "case command_step is")
  (setf *command-step* 0))

(defun command-step ()
  (emit "when" *command-step* "=>")
  (emit "command_step <=" (incf *command-step*) ";"))

(defun last-command-step ()
  (emit "when" *command-step* "=>")
  (emit "cpu_state <= read_command_state;"))

(defun command-end ()
  (emit-lines "when others =>"
              "command_step <= command_step;"
              "end case;"))

(defun absolute ()
  (command-step)
  (emit-lines "address <= pc;")
  (command-step)
  (command-step)
  (emit-lines "working(7 downto 0) <= q;"
              "pc <= pc + 1;"
              "address <= address + 1;")
  (command-step)
  (command-step)
  (emit-lines "pc <= pc + 1;"
              "address <= q & working(7 downto 0);")
  (command-step))

(defun absolute-x ()
  (command-step)
  (emit-lines "address <= pc;")
  (command-step)
  (command-step)
  (emit-lines "working(7 downto 0) <= q;"
              "pc <= pc + 1;"
              "address <= address + 1;")
  (command-step)
  (command-step)
  (emit-lines "pc <= pc + 1;"
              "address <= (q & working(7 downto 0)) + x;")
  (command-step))

(defun absolute-y ()
  (command-step)
  (emit-lines "address <= pc;")
  (command-step)
  (command-step)
  (emit-lines "working(7 downto 0) <= q;"
              "pc <= pc + 1;"
              "address <= address + 1;")
  (command-step)
  (command-step)
  (emit-lines "pc <= pc + 1;"
              "address <= (q & working(7 downto 0)) + y;")
  (command-step))

(defun immediate ()
  (command-step)
  (emit-lines "address <= pc;")
  (command-step))

(defun indirect ()
  (command-step)
  (emit-lines "address <= pc;"
              "pc <= pc + 1;")
  (command-step)
  (command-step)
  (emit-lines "working(7 downto 0) <= q;"
              "address <= address + 1;")
  (command-step)
  (command-step)
  (emit-lines "address <= q & working(7 downto 0);")
  (command-step)
  (command-step)
  (emit-lines "working(7 downto 0) <= q;"
              "address <= address + 1;")
  (command-step)
  (emit-lines "address <= q & working(7 downto 0);")
  (command-step))

(defun indirect-x ()
  (command-step)
  (emit-lines "address <= pc;"
              "pc <= pc + 1;")
  (command-step)
  (command-step)
  (emit-lines "address <= x\"00\" & (q + x);")
  (command-step)
  (command-step)
  (emit-lines "working(7 downto 0) <= q;"
              "address <= address + 1;")
  (command-step)
  (command-step)
  (emit-lines "address <= q & working(7 downto 0);")
  (command-step))

(defun indirect-y ()
  (command-step)
  (emit-lines "address <= pc;"
              "pc <= pc + 1;")
  (command-step)
  (command-step)
  (emit-lines "address <= x\"00\" & q;")
  (command-step)
  (command-step)
  (emit-lines "working(7 downto 0) <= q;"
              "address <= address + 1;")
  (command-step)
  (command-step)
  (emit-lines "address <= (q & working(7 downto 0)) + y;")
  (command-step))

(defun zeropage ()
  (command-step)
  (emit-lines "address <= pc;"
              "pc <= pc + 1;")
  (command-step)
  (command-step)
  (emit-lines "address <= x\"00\" & q;")
  (command-step))

(defun zeropage-x ()
  (command-step)
  (emit-lines "address <= pc;"
              "pc <= pc + 1;")
  (command-step)
  (command-step)
  (emit-lines "address <= (x\"00\" & q) + x;")
  (command-step))

(defun zeropage-y ()
  (command-step)
  (emit-lines "address <= pc;"
              "pc <= pc + 1;")
  (command-step)
  (command-step)
  (emit-lines "address <= (x\"00\" & q) + y;")
  (command-step))

(defun addressing-commands (name modes body-function)
  (loop for (mode-function code) in modes do
        (command (concatenate 'string
                              name " " (symbol-name mode-function))
                 code)
        (funcall mode-function)
        (funcall body-function)
        (command-end)))

(defun create-cpu ()
  (with-open-file (out "c:/data/trex6502/rtl/vhdl/t_rex_test.vhdl"
                       :direction :output
                       :if-exists :supersede)
    (setf *output* out)
    (format *output* "

-- Copyright (c) 2006 Frank Buss (fb@frank-buss.de)
-- See license.txt for license
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.ALL;

entity t_rex_test is
    port(
         clock_50mhz: in std_logic;
                      led: out unsigned(7 downto 0);
                      button: in unsigned(3 downto 0);
                      dip_switch: in unsigned(3 downto 0);
                      neg_reset: in std_logic);
end entity t_rex_test;

architecture rtl of t_rex_test is

    signal blink    : std_logic := '0';
    signal counter  : natural range 0 to 50000000 := 0;
    signal port0    : std_logic_vector(7 downto 0) := x\"00\";

    signal address  : std_logic_vector(15 downto 0);
    signal data     : std_logic_vector(7 downto 0);
    signal q        : std_logic_vector(7 downto 0);
    signal wren     : std_logic := '0';

    signal working  : std_logic_vector(15 downto 0) := x\"0200\";
    signal command  : std_logic_vector(7 downto 0);

    signal pc       : std_logic_vector(15 downto 0) := x\"0200\";
    signal sp       : std_logic_vector(15 downto 0) := x\"01ff\";
    signal accu     : std_logic_vector(7 downto 0);
    signal x        : std_logic_vector(7 downto 0);
    signal y        : std_logic_vector(7 downto 0);
    signal z_flag   : std_logic;
    signal n_flag   : std_logic;
    signal c_flag   : std_logic;
    signal v_flag   : std_logic;
    signal i_flag   : std_logic;
    signal d_flag   : std_logic;

    attribute keep: boolean;
    attribute keep of address: signal is true;
    attribute keep of data: signal is true;
    attribute keep of q: signal is true;
    attribute keep of pc: signal is true;
    attribute keep of sp: signal is true;
    attribute keep of accu: signal is true;
    attribute keep of x: signal is true;
    attribute keep of y: signal is true;
    attribute keep of z_flag: signal is true;
    attribute keep of n_flag: signal is true;

    type cpu_state_type is (
                            read_command_state,
                                              wait_for_read_state,
                                              read_memory_state,
                                              execute_command_state
                                              );

    signal cpu_state    : cpu_state_type := read_command_state;
    signal command_step : natural range 0 to 100 := 0;

begin

    instance_ram: entity ram 
    port map
    (
     address    => address(11 downto 0),
                clock   => clock_50mhz,
                data    => data,
                wren    => wren,
                q       => q
                );

    update: process(clock_50mhz, neg_reset)
    begin
        if neg_reset = '1' then
            led(2) <= '1';
        elsif rising_edge(clock_50mhz) then
            led(2) <= '0';
            if counter = 5 then
                counter <= 0;
                blink <= blink xor '1';
            else
                counter <= counter + 1;
            end if;
        end if;
    end process;
    led(7 downto 4) <= button(3 downto 0);
--  led(3 downto 2) <= dip_switch(3 downto 2);
    led(0) <= dip_switch(0) xor blink;
    
    led(1) <= q(0);

    controlProcess: process(clock_50mhz, neg_reset)
    begin
        if neg_reset = '1' then
            pc <= x\"0200\";
            sp <= x\"01ff\";
            accu <= x\"00\";
            x <= x\"00\";
            y <= x\"00\";
            z_flag <= '0';
            n_flag <= '0';
        elsif rising_edge(clock_50mhz) then
            case cpu_state is
                when read_command_state =>
                    address <= pc;
                    cpu_state <= wait_for_read_state;
                when wait_for_read_state =>
                    cpu_state <= read_memory_state;
                when read_memory_state =>
                    pc <= pc + 1;
                    command <= q;
                    command_step <= 0;
                    cpu_state <= execute_command_state;
                when execute_command_state =>
                    case command is")
                     
    (addressing-commands
     "ADC"
     '((immediate #x69)
       (zeropage #x65)
       (zeropage-x #x75)
       (absolute #x6d)
       (absolute-x #x7d)
       (absolute-y #x79)
       (indirect-x #x61)
       (indirect-y #x71))
     (lambda ()
       (last-command-step)
       (emit-lines "accu <= q + accu + c_flag;")
       (test-z-flag "q")
       (test-n-flag "q(7)")))

    (addressing-commands
     "AND"
     '((immediate #x29)
       (zeropage #x25)
       (zeropage-x #x35)
       (absolute #x2d)
       (absolute-x #x3d)
       (absolute-y #x39)
       (indirect-x #x21)
       (indirect-y #x31))
     (lambda ()
       (last-command-step)
       (emit-lines "accu <= q and accu;")
       (test-z-flag "q")
       (test-n-flag "q(7)")))

    (addressing-commands
     "ASL"
     '((zeropage #x06)
       (zeropage-x #x16)
       (absolute #x0e)
       (absolute-x #x1e))
     (lambda ()
       (command-step)
       (emit-lines "data <= q(6 downto 0) & '0';"
                   "c_flag <= q(7);"
                   "wren <= '1';")
       (command-step)
       (last-command-step)
       (emit-lines "wren <= '0';")
       (test-z-flag "data")
       (test-n-flag "data(7)")))

    (command "ASL accu" #x0a)
    (last-command-step)
    (emit-lines "accu <= accu(6 downto 0) & '0';"
                "c_flag <= accu(7);")
    (command-end)

    (command "BCC" #x90)
    (immediate)
    (last-command-step)
    (emit-lines "if c_flag = '0' then"
                "pc <= pc + q;"
                "end if;")
    (command-end)

    (command "BCS" #xb0)
    (immediate)
    (last-command-step)
    (emit-lines "if c_flag = '1' then"
                "pc <= pc + q;"
                "end if;")
    (command-end)

    (command "BEQ" #xf0)
    (immediate)
    (last-command-step)
    (emit-lines "if z_flag = '1' then"
                "pc <= pc + q;"
                "end if;")
    (command-end)

    (addressing-commands
     "BIT"
     '((zeropage #x24)
       (absolute #x2c))
     (lambda ()
       (last-command-step)
       (emit-lines "if (q and accu) = x\"00\" then z_flag <= '1'; else z_flag <= '0'; end if;"
                   "n_flag <= q(7);"
                   "v_flag <= q(6);")))

    (command "BMI" #x30)
    (immediate)
    (last-command-step)
    (emit-lines "if n_flag = '1' then"
                "pc <= pc + q;"
                "end if;")
    (command-end)

    (command "BNE" #xd0)
    (immediate)
    (last-command-step)
    (emit-lines "if z_flag = '0' then"
                "pc <= pc + q;"
                "end if;")
    (command-end)

    (command "BPL" #x10)
    (immediate)
    (last-command-step)
    (emit-lines "if n_flag = '0' then"
                "pc <= pc + q;"
                "end if;")
    (command-end)

    (command "BRK" #x00)
    ;; todo
    (last-command-step)
    (command-end)

    (command "BVC" #x50)
    (immediate)
    (last-command-step)
    (emit-lines "if v_flag = '0' then"
                "pc <= pc + q;"
                "end if;")
    (command-end)

    (command "BVS" #x70)
    (immediate)
    (last-command-step)
    (emit-lines "if v_flag = '1' then"
                "pc <= pc + q;"
                "end if;")
    (command-end)

    (command "CLC" #x18)
    (immediate)
    (last-command-step)
    (emit-lines "c_flag <= '0';")
    (command-end)

    (command "CLD" #xd8)
    (immediate)
    (last-command-step)
    (emit-lines "d_flag <= '0';")
    (command-end)

    (command "CLI" #x58)
    (immediate)
    (last-command-step)
    (emit-lines "i_flag <= '0';")
    (command-end)

    (command "CLV" #xb8)
    (immediate)
    (last-command-step)
    (emit-lines "v_flag <= '0';")
    (command-end)

    (addressing-commands
     "CMP"
     '((immediate #xc9)
       (zeropage #xc5)
       (zeropage-x #xd5)
       (absolute #xcd)
       (absolute-x #xdd)
       (absolute-y #xd9)
       (indirect-x #xc1)
       (indirect-y #xd1))
     (lambda ()
       (last-command-step)
       (emit-lines "if accu < q then"
                   "c_flag <= '1';"
                   "else"
                   "c_flag <= '0';"
                   "end if;")))
    ;; todo

    (addressing-commands
     "CPX"
     '((immediate #xe0)
       (zeropage #xe4)
       (absolute #xec))
     (lambda ()
       (last-command-step)
       (emit-lines "if x < q then"
                   "c_flag <= '1';"
                   "else"
                   "c_flag <= '0';"
                   "end if;")))
    ;; todo

    (addressing-commands
     "CPY"
     '((immediate #xc0)
       (zeropage #xc4)
       (absolute #xcc))
     (lambda ()
       (last-command-step)
       (emit-lines "if y < q then"
                   "c_flag <= '1';"
                   "else"
                   "c_flag <= '0';"
                   "end if;")))
    ;; todo

    (addressing-commands
     "DEC"
     '((zeropage #xc6)
       (zeropage-x #xd6)
       (absolute #xce)
       (absolute-x #xde))
     (lambda ()
       (command-step)
       (emit-lines "data <= q - 1;"
                   "wren <= '1';")
       (command-step)
       (last-command-step)
       (emit-lines "wren <= '0';")
       (test-z-flag "data")
       (test-n-flag "data(7)")))

    (command "DEX" #xca)
    (command-step)
    (emit-lines "x <= x - 1;")
    (last-command-step)
    (test-z-flag "x")
    (test-n-flag "x(7)")
    (command-end)

    (command "DEY" #x88)
    (command-step)
    (emit-lines "y <= y - 1;")
    (last-command-step)
    (test-z-flag "y")
    (test-n-flag "y(7)")
    (command-end)

    (addressing-commands
     "EOR"
     '((immediate #x49)
       (zeropage #x45)
       (zeropage-x #x55)
       (absolute #x4d)
       (absolute-x #x5d)
       (absolute-y #x59)
       (indirect-x #x41)
       (indirect-y #x51))
     (lambda ()
       (last-command-step)
       (emit-lines "accu <= q xor accu;")
       (test-z-flag "q")
       (test-n-flag "q(7)")))

    (addressing-commands
     "INC"
     '((zeropage #xe6)
       (zeropage-x #xf6)
       (absolute #xee)
       (absolute-x #xfe))
     (lambda ()
       (command-step)
       (emit-lines "data <= q + 1;"
                   "wren <= '1';")
       (command-step)
       (last-command-step)
       (emit-lines "wren <= '0';")
       (test-z-flag "data")
       (test-n-flag "data(7)")))

    (command "INX" #xe8)
    (command-step)
    (emit-lines "x <= x + 1;")
    (last-command-step)
    (test-z-flag "x")
    (test-n-flag "x(7)")
    (command-end)

    (command "INY" #xc8)
    (command-step)
    (emit-lines "y <= y + 1;")
    (last-command-step)
    (test-z-flag "y")
    (test-n-flag "y(7)")
    (command-end)

    (addressing-commands
     "JMP"
     '((absolute #x4c)
       (indirect #x6c))
     (lambda ()
       (last-command-step)
       (emit-lines "pc <= address;")))

    (command "JSR" #x20)
    (absolute)
    (command-step)
    (emit-lines "working <= address;"
                "address <= sp;"
                "sp <= sp - 1;"
                "data <= pc(7 downto 0);"
                "wren <= '1';")
    (command-step)
    (emit-lines "address <= address - 1;"
                "sp <= sp - 1;"
                "data <= pc(15 downto 8);")
    (last-command-step)
    (emit-lines "wren <= '0';"
                "pc <= working;")
    (command-end)

    (addressing-commands
     "LDA"
     '((immediate #xa9)
       (zeropage #xa5)
       (zeropage-x #xb5)
       (absolute #xad)
       (absolute-x #xbd)
       (absolute-y #xb9)
       (indirect-x #xa1)
       (indirect-y #xb1))
     (lambda ()
       (last-command-step)
       (emit-lines "accu <= q;")
       (test-z-flag "q")
       (test-n-flag "q(7)")))

    (addressing-commands
     "LDX"
     '((immediate #xa2)
       (zeropage #xa6)
       (zeropage-y #xb6)
       (absolute #xae)
       (absolute-y #xbe))
     (lambda ()
       (last-command-step)
       (emit-lines "x <= q;")
       (test-z-flag "q")
       (test-n-flag "q(7)")))

    (addressing-commands
     "LDY"
     '((immediate #xa0)
       (zeropage #xa4)
       (zeropage-x #xb4)
       (absolute #xac)
       (absolute-x #xbc))
     (lambda ()
       (last-command-step)
       (emit-lines "x <= q;")
       (test-z-flag "q")
       (test-n-flag "q(7)")))

    (addressing-commands
     "LSR"
     '((zeropage #x46)
       (zeropage-x #x56)
       (absolute #x4e)
       (absolute-x #x5e))
     (lambda ()
       (command-step)
       (emit-lines "data <= '0' & q(7 downto 1);"
                   "c_flag <= q(0);"
                   "wren <= '1';")
       (command-step)
       (last-command-step)
       (emit-lines "wren <= '0';")
       (test-z-flag "data")
       (test-n-flag "data(7)")))

    (command "LSR accu" #x4a)
    (last-command-step)
    (emit-lines "accu <= '0' & accu(7 downto 1);"
                "c_flag <= accu(0);")
    (command-end)

    (command "NOP" #xea)
    (last-command-step)
    (command-end)

    (addressing-commands
     "ORA"
     '((immediate #x09)
       (zeropage #x05)
       (zeropage-x #x15)
       (absolute #x0d)
       (absolute-x #x1d)
       (absolute-y #x19)
       (indirect-x #x01)
       (indirect-y #x11))
     (lambda ()
       (last-command-step)
       (emit-lines "accu <= q or accu;")
       (test-z-flag "q")
       (test-n-flag "q(7)")))

    (command "PHA" #x48)
    (command-step)
    (emit-lines "address <= sp;"
                "data <= accu;"
                "sp <= sp - 1;"
                "wren <= '1';")
    (last-command-step)
    (emit-lines "wren <= '0';")
    (command-end)

    (command "PHP" #x08)
    (command-step)
    (emit-lines "address <= sp;"
                "data <= n_flag & z_flag & c_flag & i_flag & d_flag & v_flag & \"00\";"
                "sp <= sp - 1;"
                "wren <= '1';")
    (last-command-step)
    (emit-lines "wren <= '0';")
    (command-end)

    (command "PLA" #x68)
    (command-step)
    (emit-lines "address <= sp;"
                "sp <= sp + 1;")
    (command-step)
    (last-command-step)
    (emit-lines "accu <= q;")
    (command-end)

    (command "PLP" #x28)
    (command-step)
    (emit-lines "address <= sp;"
                "sp <= sp + 1;")
    (command-step)
    (last-command-step)
    (emit-lines "n_flag <= q(7);"
                "z_flag <= q(6);"
                "c_flag <= q(5);"
                "i_flag <= q(4);"
                "d_flag <= q(3);"
                "v_flag <= q(2);")
    (command-end)

    (addressing-commands
     "ROL"
     '((zeropage #x26)
       (zeropage-x #x36)
       (absolute #x2e)
       (absolute-x #x3e))
     (lambda ()
       (command-step)
       (emit-lines "data <= q(6 downto 0) & c_flag;"
                   "c_flag <= q(7);"
                   "wren <= '1';")
       (command-step)
       (last-command-step)
       (emit-lines "wren <= '0';")
       (test-z-flag "data")
       (test-n-flag "data(7)")))

    (command "ROL accu" #x2a)
    (last-command-step)
    (emit-lines "accu <= accu(6 downto 0) & c_flag;"
                "c_flag <= accu(7);")
    (command-end)

    (addressing-commands
     "ROR"
     '((zeropage #x66)
       (zeropage-x #x76)
       (absolute #x6e)
       (absolute-x #x7e))
     (lambda ()
       (command-step)
       (emit-lines "data <= c_flag & q(7 downto 1);"
                   "c_flag <= q(0);"
                   "wren <= '1';")
       (command-step)
       (last-command-step)
       (emit-lines "wren <= '0';")
       (test-z-flag "data")
       (test-n-flag "data(7)")))

    (command "ROR accu" #x6a)
    (last-command-step)
    (emit-lines "accu <= c_flag & accu(7 downto 1);"
                "c_flag <= accu(0);")
    (command-end)

    (command "RTI" #x40)
    (last-command-step)
    (command-end)
    ;; todo

    (command "RTS" #x60)
    (command-step)
    (emit-lines "address <= sp;"
                "sp <= sp + 1;")
    (command-step)
    (command-step)
    (emit-lines "working(7 downto 0) <= q;"
                "sp <= sp + 1;")
    (command-step)
    (last-command-step)
    (emit-lines "pc <= q & working(7 downto 0);")
    (command-end)

    (addressing-commands
     "SBC"
     '((immediate #xe9)
       (zeropage #xe5)
       (zeropage-x #xf5)
       (absolute #xed)
       (absolute-x #xfd)
       (absolute-y #xf9)
       (indirect-x #xe1)
       (indirect-y #xf1))
     (lambda ()
       (last-command-step)
       (emit-lines "accu <= accu - q - c_flag;")
       (test-z-flag "q")
       (test-n-flag "q(7)")))

    (command "SEC" #x38)
    (immediate)
    (last-command-step)
    (emit-lines "c_flag <= '1';")
    (command-end)

    (command "SED" #xf8)
    (immediate)
    (last-command-step)
    (emit-lines "d_flag <= '1';")
    (command-end)

    (command "SEI" #x78)
    (immediate)
    (last-command-step)
    (emit-lines "i_flag <= '1';")
    (command-end)

    (addressing-commands
     "STA"
     '((zeropage #x85)
       (zeropage-x #x95)
       (absolute #x8d)
       (absolute-x #x9d)
       (absolute-y #x99)
       (indirect-x #x81)
       (indirect-y #x91))
     (lambda ()
       (command-step)
       (emit-lines "data <= accu;"
                   "wren <= '1';")
       (last-command-step)
       (emit-lines "wren <= '0';")))

    (addressing-commands
     "STX"
     '((zeropage #x86)
       (zeropage-y #x96)
       (absolute #x8e))
     (lambda ()
       (command-step)
       (emit-lines "data <= x;"
                   "wren <= '1';")
       (last-command-step)
       (emit-lines "wren <= '0';")))

    (addressing-commands
     "STY"
     '((zeropage #x84)
       (zeropage-y #x94)
       (absolute #x8c))
     (lambda ()
       (command-step)
       (emit-lines "data <= y;"
                   "wren <= '1';")
       (last-command-step)
       (emit-lines "wren <= '0';")))

    (command "TAX" #xaa)
    (last-command-step)
    (emit-lines "accu <= x;")
    (test-z-flag "x")
    (test-n-flag "x(7)")
    (command-end)

    (command "TAX" #xa8)
    (last-command-step)
    (emit-lines "accu <= y;")
    (test-z-flag "y")
    (test-n-flag "y(7)")
    (command-end)

    (command "TSX" #xba)
    (last-command-step)
    (emit-lines "x <= sp(7 downto 0);")
    (test-z-flag "sp(7 downto 0)")
    (test-n-flag "sp(7)")
    (command-end)

    (command "TXA" #x8a)
    (last-command-step)
    (emit-lines "x <= accu;")
    (test-z-flag "accu")
    (test-n-flag "accu(7)")
    (command-end)

    (command "TXS" #x9a)
    (last-command-step)
    (emit-lines "sp(7 downto 0) <= x;")
    (test-z-flag "x")
    (test-n-flag "x(7)")
    (command-end)

    (command "TYA" #x98)
    (last-command-step)
    (emit-lines "accu <= y;")
    (test-z-flag "y")
    (test-n-flag "y(7)")
    (command-end)

    (format *output* "
                        when others =>
                            -- unknown command, read next command
                            cpu_state <= read_command_state;
                    end case;
            end case;
        end if;
    end process;

end architecture rtl;
")))

(create-cpu)
